import ConfirmPopup from '@components/common/ConfirmPopup'
import Popup from '@components/common/Popup'
import TextInput from '@components/common/TextInput'
import Textarea from '@components/common/Textarea'
import { createForm } from '@felte/solid'
import { validator } from '@felte/validator-yup'
import useLanguage from '@hooks/useLanguage'
import {
  IconAddressBook,
  IconCirclePlus,
  IconEditCircle,
  IconFileDescription,
  IconInfoCircle,
  IconVector,
} from '@tabler/icons-solidjs'
import {
  createComponent,
  createEffect,
  createSignal,
  For,
  Show,
  untrack,
} from 'solid-js'
import { v4 as uuidv4 } from 'uuid'
import * as yup from 'yup'
import AreaItem from './AreaItem'

/**
 * Returns a Yup schema object for validating an area object.
 *
 * @param {Object} language - An object containing the language settings.
 * @param {Function} isRequired - A function that returns a validation message for required fields.
 * @return {Object} A Yup schema object with two required fields: name and description.
 */
const areaSchema = (language, isRequired) =>
  yup.object({
    name: yup
      .string()
      .min(3, language.message['MIN_THREE_CHAR'])
      .required(isRequired(language.ui.areaName)),
    desc: yup
      .string()
      .min(3, language.message['MIN_THREE_CHAR'])
      .required(isRequired(language.ui.areaName)),
  })

export default function AreaAdd(props) {
  const [openModal, setOpenModal] = createSignal(false)
  const [dataArea, setDataArea] = createSignal([])
  const [editMode, setEditMode] = createSignal(false)
  const { language, isRequired } = useLanguage()
  const { form, reset, data, setData, errors, createSubmitHandler } =
    createForm({
      extend: [validator({ schema: areaSchema(language, isRequired) })],
      onSubmit: async (values) => {
        values.id = uuidv4()
        values.isCreate = true
        setDataArea((prev) => [...prev, values])
        onModalClose()
      },
    })

  createEffect(() => {
    if (props.value && dataArea().length === 0) {
      console.log(props.value)
      untrack(() => setDataArea(props.value))
    }
  })

  const onModalClose = () => {
    setOpenModal(false), reset(), setEditMode(false)
  }

  const onOpenModal = () => {
    setOpenModal(true)
  }

  const onConfirmDelete = (delId) => {
    setDataArea((prev) => {
      prev = prev.filter((item) => item.id !== delId)
      return [...prev]
    })
    props.setData('areas', dataArea())
  }

  const onClickEditItem = (id) => {
    const editItem = dataArea().find((item) => item.id === id)
    setData(editItem)
    setEditMode(true)
    setOpenModal(true)
  }

  const onClickUpdateItem = createSubmitHandler({
    onSubmit: (values) => {
      setDataArea((prev) => {
        prev = prev.map((item) => {
          if (item.id === values.id) {
            return values
          }
          return item
        })
        return [...prev]
      })
      props.setData('areas', dataArea())
      reset()
      setEditMode(false)
      onModalClose()
    },
  })

  const onClickDeleteItem = (delId) => {
    createComponent(ConfirmPopup, {
      title: language?.message['CONFIRM_DELETE'],
      children: language?.message['CONFIRM_DELETE_NOTE'],
      deleteId: delId,
      onConfirm: onConfirmDelete,
    })
  }

  return (
    <div class="form-control mb-3">
      <div class="join join-vertical">
        <div
          class="flex items-center justify-between bg-base-200 border border-gray-300 h-12 px-3 join-item"
          classList={{ 'border-red-500': props.error }}
        >
          <label class="label justify-start gap-2">
            <IconVector size={18} />
            <span class="label-text font-bold whitespace-nowrap">
              {language.ui.areas}
            </span>
            <Show when={props.error}>
              <div
                class="tooltip tooltip-right tooltip-error before:text-white text-red-500"
                data-tip={props.error}
              >
                <IconInfoCircle size={18} />
              </div>
            </Show>
          </label>
          <button
            type="button"
            class="btn btn-ghost btn-circle btn-xs text-green-400 hover:bg-green-100 hover:text-green-500"
            onClick={onOpenModal}
          >
            <IconCirclePlus size={18} />
          </button>
        </div>
        <div
          class="scroll-shadow-vertical no-scrollbar border border-gray-300 join-item"
          classList={{ 'border-red-500': props.error }}
        >
          <div class="p-3 grid grid-cols-1 md:grid-cols-2 lg:grid-cols-3 gap-4">
            <Show when={dataArea().length > 0} fallback={language.ui.empty}>
              <For each={dataArea()}>
                {(item, index) => (
                  <AreaItem
                    data={item}
                    formName={props.name}
                    num={index()}
                    onDelete={onClickDeleteItem}
                    onEdit={onClickEditItem}
                  />
                )}
              </For>
            </Show>
          </div>
        </div>
      </div>
      <Popup
        icon={
          editMode() ? (
            <IconEditCircle size={20} class="text-blue-500" />
          ) : (
            <IconCirclePlus size={20} class="text-green-500" />
          )
        }
        title={editMode() ? language.ui.editArea : language.ui.addArea}
        titleClass="text-lg"
        openModal={openModal()}
        onModalClose={onModalClose}
        class="!w-6/12 !max-w-5xl"
      >
        <form autocomplete="off" use:form>
          <div class="modal-body mt-4">
            <TextInput
              icon={IconAddressBook}
              name="name"
              label={language.ui.areaName}
              placeholder={language.ui.areaName}
              value={data('name')}
              error={errors('name')}
            />
            <Textarea
              icon={IconFileDescription}
              name="desc"
              label={language.ui.areaDesc}
              placeholder={language.ui.areaDesc}
              value={data('desc')}
              error={errors('desc')}
            />
          </div>
          <div class="modal-action">
            <Show
              when={editMode()}
              fallback={
                <button type="submit" class="btn btn-primary">
                  {language.ui.save}
                </button>
              }
            >
              <button
                type="submit"
                class="btn btn-primary"
                onClick={onClickUpdateItem}
              >
                {language.ui.update}
              </button>
            </Show>
            <button type="button" class="btn btn-ghost" onClick={onModalClose}>
              {language.ui.cancel}
            </button>
          </div>
        </form>
      </Popup>
    </div>
  )
}
