import ViewSwitch, { VIEWDATA } from '@components/ViewSwitch'
import useLanguage from '@hooks/useLanguage'
import * as icons from '@tabler/icons-solidjs'
import {
  For,
  createComponent,
  createEffect,
  createResource,
  createSignal,
} from 'solid-js'

import { getAllHouse } from '@api/house'
import ConfirmPopup from '@components/common/ConfirmPopup'
import Pagination from '@components/common/Pagination'
import { A, useNavigate } from '@solidjs/router'
import {
  IconHome,
  IconPencil,
  IconSquareRoundedPlus,
  IconTrash,
} from '@tabler/icons-solidjs'
import { Helpers } from '@utils/helper'
import { Dynamic } from 'solid-js/web'
import './house.scss'

const PAGE_SIZE = [10, 50, 100]

const getPaginationText = (total, size, page) => {
  const start = (page - 1) * size + 1
  const end = Math.min(start + size - 1, total)

  return `${start} - ${end} / ${total}`
}

const fetchHouses = async ({ page, pageSize }) => {
  const response = await getAllHouse({ page, pageSize })
  return response
}

export default function House() {
  const { language } = useLanguage()
  const navigate = useNavigate()
  const [pageSize, setPageSize] = createSignal(PAGE_SIZE[0])
  const [currentPage, setCurrentPage] = createSignal(1)
  const [view, setView] = createSignal(VIEWDATA['list'])
  const [record, setRecord] = createSignal(null)
  const [totalRecord, setTotalRecord] = createSignal(0)
  const [houses] = createResource(
    () => ({ page: currentPage(), pageSize: pageSize() }),
    fetchHouses,
  )

  createEffect(() => {
    if (houses()) {
      setRecord(houses()?.data?.list)
      setTotalRecord(houses()?.data?.total)
    }
  })

  const onClickEdit = (editId) => {
    navigate(Helpers.getRoutePath('edit-location').replace(':id', editId), {
      replace: true,
    })
  }

  const onConfirmDelete = (deleteId) => {
    console.log(deleteId)
  }

  const onClickDelete = (deleteId) => {
    createComponent(ConfirmPopup, {
      title: language?.message['CONFIRM_DELETE'],
      children: language?.message['CONFIRM_DELETE_NOTE'],
      deleteId,
      onConfirm: onConfirmDelete,
    })
  }

  const onSetPageSize = (pageSize) => {
    setPageSize(pageSize)
    setCurrentPage(1)
  }

  return (
    <div class="house">
      <div class="flex items-center gap-2 mb-5 text-xl">
        <span class="text-secondary">
          <IconHome size={30} />
        </span>
        {language.ui.house}
      </div>
      <div class="page-topbar flex justify-between mb-4">
        <ViewSwitch switchView={setView} />
        <A
          href="/location/create"
          class="btn btn-success text-white hover:text-white btn-sm"
        >
          <IconSquareRoundedPlus size={15} />
          {language.ui.createNew}
        </A>
      </div>
      <div
        class="view-layout scroll-shadow-horizontal no-scrollbar"
        classList={{
          'view-list': view() === VIEWDATA['list'],
          'view-grid': view() === VIEWDATA['grid'],
        }}
      >
        <div class="view-table">
          <div class="row view-head">
            <div class="col w-1/12">{language.table.columnName.no}</div>
            <div class="col w-1/12">{language.table.columnName.icon}</div>
            <div class="col w-4/12">{language.table.columnName.name}</div>
            <div class="col w-4/12">{language.table.columnName.address}</div>
            <div class="col w-2/12">{language.table.columnName.action}</div>
          </div>
          <For each={record()}>
            {(item, idx) => (
              <div class="row view-item">
                <div class="col hide w-1/12">
                  {pageSize() * currentPage() - (pageSize() - idx() - 1)}
                </div>
                <div class="col symbol w-1/12">
                  <Dynamic component={icons[item?.icon]} class="w-7 h-7" />
                </div>
                <div class="col w-4/12">{item?.name}</div>
                <div class="col w-4/12">{item?.address}</div>
                <div class="col actionbar w-2/12">
                  <button
                    class="btn btn-ghost btn-sm px-1 text-blue-500"
                    onClick={[onClickEdit, item?.id]}
                  >
                    <IconPencil size={20} />
                  </button>
                  <button
                    class="btn btn-ghost btn-sm px-1 text-red-500"
                    onClick={[onClickDelete, item?.id]}
                  >
                    <IconTrash size={20} />
                  </button>
                </div>
              </div>
            )}
          </For>
        </div>
      </div>
      <div class="page-botbar flex max-xs:flex-col justify-between items-center gap-2 mt-5">
        <div class="bar-left flex gap-2 justify-start items-center">
          <div class="py-2 px-3 border rounded-lg border-gray-300 leading-none">
            <span>
              {language.ui.showing}:{' '}
              {getPaginationText(totalRecord(), pageSize(), currentPage())}
            </span>
          </div>
          <div class="dropdown dropdown-top dropdown-end">
            <div
              tabindex="0"
              role="button"
              class="btn btn-sm border border-gray-300"
            >
              {pageSize()}
            </div>
            <ul
              tabindex="0"
              class="dropdown-content menu bg-base-100 rounded-box z-[1] w-32 p-2 shadow mb-2"
            >
              <For each={PAGE_SIZE}>
                {(pageSize) => (
                  <li>
                    <a onClick={[onSetPageSize, pageSize]}>{pageSize}</a>
                  </li>
                )}
              </For>
            </ul>
          </div>
        </div>
        <div class="bar-right">
          <Pagination
            currentPage={currentPage}
            totalCount={totalRecord}
            pageSize={pageSize}
            onPageChange={setCurrentPage}
          />
        </div>
      </div>
    </div>
  )
}
