import { getHouseDetail, postCreateHouse, putUpdateHouse } from '@api/house'
import AreaAdd from '@components/AreaAdd'
import TextInput from '@components/common/TextInput'
import { createForm } from '@felte/solid'
import { validator } from '@felte/validator-yup'
import useLanguage from '@hooks/useLanguage'
import useToast from '@hooks/useToast'
import { A, useNavigate, useParams } from '@solidjs/router'
import {
  IconAddressBook,
  IconIcons,
  IconLocationBolt,
  IconMapPinPlus,
  IconTag,
} from '@tabler/icons-solidjs'
import { Helpers } from '@utils/helper'
import { createEffect, createResource, Show } from 'solid-js'
import * as yup from 'yup'

const houseSchema = (language, isRequired) =>
  yup.object({
    icon: yup
      .string()
      .min(3, language.message['MIN_THREE_CHAR'])
      .required(isRequired(language.ui.houseIcon)),
    name: yup
      .string()
      .min(3, language.message['MIN_THREE_CHAR'])
      .required(isRequired(language.ui.houseName)),
    address: yup
      .string()
      .min(3, language.message['MIN_THREE_CHAR'])
      .required(isRequired(language.ui.houseAddress)),
    areas: yup
      .array()
      .min(1, isRequired(language.ui.areas))
      .required(isRequired(language.ui.areas)),
  })

const fetchHouses = async (id) => {
  if (id) {
    const response = await getHouseDetail(id)
    if (response.status === 200) {
      return response.data
    }
    return response
  }

  return null
}

export default function HouseCreate() {
  const { language, isRequired } = useLanguage()
  const notify = useToast()
  const navigate = useNavigate()
  const { id } = useParams()
  const [house] = createResource(id, fetchHouses)
  const { form, data, setData, errors } = createForm({
    extend: [validator({ schema: houseSchema(language, isRequired) })],
    onSubmit: async (values) => {
      const call = id ? putUpdateHouse : postCreateHouse
      const result = {
        ...values,
        areas: values.areas.map((item) =>
          id
            ? { ...item }
            : {
                name: item.name,
                desc: item.desc,
              },
        ),
      }
      console.log(values.areas)
      const resp = await call(result)
      return resp
    },
    onSuccess: (resp) => {
      if (resp.status === 200) {
        notify.success({
          title: language.ui.success,
          description:
            language.message[resp.data] ||
            language.message['CREATE_HOUSE_SUCCESS'],
        })
        // navigate(Helpers.getRoutePath('location'), { replace: true })
      }
    },
    onError: (error) => {
      notify.error({
        title: language.ui.error,
        description:
          language.message[error.data] || language.message['API_CALL_FAIL'],
        closable: true,
      })
    },
  })

  createEffect(() => {
    if (id && house()) {
      setData(house())
    }
  })

  return (
    <div class="house-create">
      <div class="text-sm breadcrumbs mb-2">
        <ul>
          <li>
            <A href={Helpers.getRoutePath('location')}>{language.ui.house}</A>
          </li>
          <li>{id ? language.ui.editHouse : language.ui.newHouse}</li>
        </ul>
      </div>
      <div class="flex items-center gap-2 mb-5 text-xl">
        <span class="text-secondary">
          {id ? <IconLocationBolt size={30} /> : <IconMapPinPlus size={30} />}
        </span>
        {id ? language.ui.editHouse : language.ui.newHouse}
      </div>
      <div class="card w-full bg-base-100 shadow-lg border border-gray-200">
        <div class="card-body">
          <form autoComplete="off" use:form>
            <Show when={id}>
              <input type="hidden" name="id" value={id} />
            </Show>
            <div class="grid gap-4 grid-cols-1 lg:grid-cols-2">
              <div class="col-auto">
                <TextInput
                  icon={IconIcons}
                  name="icon"
                  label={language.ui.houseIcon}
                  labelClass="md:w-40"
                  placeholder={language.ui.houseIcon}
                  value={data('icon')}
                  error={errors('icon')}
                >
                  <div class="label">
                    <a
                      class="label-text link link-info"
                      href="https://tabler.io/icons"
                      target="_blank"
                    >
                      {language.ui.findIconHere}
                    </a>
                  </div>
                </TextInput>
              </div>
              <div class="col-auto">
                <TextInput
                  icon={IconTag}
                  name="name"
                  label={language.ui.houseName}
                  labelClass="md:w-40"
                  placeholder={language.ui.houseName}
                  value={data('name')}
                  error={errors('name')}
                />
              </div>
              <div class="col-auto lg:col-span-2">
                <TextInput
                  icon={IconAddressBook}
                  name="address"
                  label={language.ui.houseAddress}
                  labelClass="md:w-40"
                  placeholder={language.ui.houseAddress}
                  value={data('address')}
                  error={errors('address')}
                />
              </div>
              <div class="col-auto lg:col-span-2">
                <AreaAdd
                  name="areas"
                  value={data('areas')}
                  setData={setData}
                  error={
                    errors('areas') &&
                    Helpers.clearArrayWithNullObject(errors('areas'))
                  }
                />
              </div>
            </div>
            <div class="card-actions">
              <button type="submit" class="btn btn-primary">
                {id ? language.ui.update : language.ui.create}
              </button>
              <A href={Helpers.getRoutePath('location')} class="btn btn-ghost">
                {language.ui.cancel}
              </A>
            </div>
          </form>
        </div>
      </div>
    </div>
  )
}
